// Copyright (C) 2012 MicroStrain, Inc.

// 2400_sdkDlg.h : header file


#pragma once
#include "afxcmn.h"
#include "afxwin.h"

#include "clistbox.h"

static unsigned int CBR_921600 = 921600;

// CsdkDlg dialog
class CsdkDlg : public CDialog
{
// Construction
public:
	CsdkDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
	enum { IDD = IDD_SDK_DIALOG };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support


// Implementation
protected:
	HICON m_hIcon;

	// Generated message map functions
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	DECLARE_MESSAGE_MAP()
public:
	virtual BOOL OnInitDialog();
private:
	/************************
	*	make_port_string()
	*	make_port_string() is called to convert the integer value of a serial port
	*	to the correct string format.
	*	PRE:	uiPort is an unsigned integer port number
	*	POST:	a correctly formatted CString object has been returned with the port's
	*				number already in it.
	************************/
	CString make_port_string(unsigned int uiPort);
private:
	CEdit m_cePort;								//	Port number
	CSpinButtonCtrl m_csbcPort;					//	Port number spin control
	CCListBox m_clbConsole;						//	Console
	CSpinButtonCtrl m_csbcBase_eeprom_location;	//	Base station EEPROM location spin control
	CSpinButtonCtrl m_csbcBase_eeprom_value;	//	Base station EEPROM value spin control
	CSpinButtonCtrl m_csbcNode_eeprom_location;	//	Node EEPROM location spin control
	CSpinButtonCtrl m_csbcNode_eeprom_value;	//	Node EEPROM value spin control
	CSpinButtonCtrl m_csbcNode;					//	Node address spin control
	CSpinButtonCtrl m_csbcPage;					//	Download page spin control
	CSpinButtonCtrl m_csbcTrigger;				//	Trigger ID spin control
	CEdit m_editCmd_info;						//	Command info output
	CButton m_BaudCheck;						//	Checkbox to enable 921600 Baud Rate
public:

	/************************
	*	OnBnClickedButtonPing()
	*	OnBnClickedButtonPing() is called when the user selects the ping button in the
	*	base station commands group box.  Please, investigate the implementation of each
	*	individual function for a better understanding of each command.
	************************/	
	afx_msg void OnBnClickedButtonPing();

	/************************
	*	OnEnKillfocusEditEepromLocation()
	*	OnEnKillfocusEditEepromLocation() checks the range of the base stationeeprom location
	*	when focus on the eeprom location edit box has lost focus.
	*	PRE:	none
	*	POST:	if the value in the edit control is greater than the maximum
	*				range the value is adjusted to the maximum range value.  Likewise,
	*				if the value in the edit control is less tahn the maximum
	*				range the value is adjusted to the minimum range value.
	************************/
	afx_msg void OnEnKillfocusEditEepromLocation();

	/************************
	*	OnEnKillfocusEditEepromValue()
	*	OnEnKillfocusEditEepromValue() checks the range of the base station eeprom value
	*	when focus on the eeprom value edit box has lost focus.
	*	PRE:	none
	*	POST:	if the value in the edit control is greater than the maximum
	*				range the value is adjusted to the maximum range value.  Likewise,
	*				if the value in the edit control is less tahn the maximum
	*				range the value is adjusted to the minimum range value.
	************************/	
	afx_msg void OnEnKillfocusEditEepromValue();

	/************************
	*	OnEnKillfocusEditPort()
	*	OnEnKillfocusEditPort() is called when the area the user enters the port number
	*	at on the dialog box loses focus.  It is used to perform a range test.  If the
	*	the port number is out of range, either greater than PORT_NUM_HI or lower than
	*	PORT_NUM_LO, the item will be set to PORT_NUM_HI or PORT_NUM_LO depending on
	*	which end of the range is most least broken.
	************************/
	afx_msg void OnEnKillfocusEditPort();

	/************************
	*	OnBnClickedButtonBsReadEeprom()
	*	OnBnClickedButtonBsReadEeprom() is called when the user selects the read eeprom
	*	button in the base station group box.  Please, investigate the implementation of each
	*	individual function for a better understanding of each command.
	************************/
	afx_msg void OnBnClickedButtonBsReadEeprom();

	/************************
	*	OnBnClickedButtonBsWriteEeprom()
	*	OnBnClickedButtonBsWriteEeprom() is called when the user selects the write eeprom
	*	button in the base station group box.  Please, investigate the implementation of each
	*	individual function for a better understanding of each command.
	************************/
	afx_msg void OnBnClickedButtonBsWriteEeprom();

	/************************
	*	OnEnKillfocusEditNdEepromLocation()
	*	OnEnKillfocusEditNdEepromLocation() checks the range of the node eeprom location
	*	when focus on the eeprom location edit box has lost focus.
	*	PRE:	none
	*	POST:	if the value in the edit control is greater than the maximum
	*				range the value is adjusted to the maximum range value.  Likewise,
	*				if the value in the edit control is less than the maximum
	*				range the value is adjusted to the minimum range value.
	************************/
	afx_msg void OnEnKillfocusEditNdEepromLocation();

	/************************
	*	OnEnKillfocusEditNdEepromValue()
	*	OnEnKillfocusEditNdEepromValue() checks the range of the node eeprom value
	*	when focus on the eeprom value edit box has lost focus.
	*	PRE:	none
	*	POST:	if the value in the edit control is greater than the maximum
	*				range the value is adjusted to the maximum range value.  Likewise,
	*				if the value in the edit control is less tahn the maximum
	*				range the value is adjusted to the minimum range value.
	************************/
	afx_msg void OnEnKillfocusEditNdEepromValue();

	/************************
	*	OnBnClickedButtonNdReadEeprom()
	*	OnBnClickedButtonNdReadEeprom() is called when the user selects the read eeprom
	*	button in the node group box.  Please, investigate the implementation of each
	*	individual function for a better understanding of each command.
	************************/
	afx_msg void OnBnClickedButtonNdReadEeprom();

	/************************
	*	OnEnKillfocusEditNode()
	*	OnEnKillfocusEditNode() checks the range of the node'e address
	*	when focus on the node's address edit box has lost focus.
	*	PRE:	none
	*	POST:	if the value in the edit control is greater than the maximum
	*				range the value is adjusted to the maximum range value.  Likewise,
	*				if the value in the edit control is less tahn the maximum
	*				range the value is adjusted to the minimum range value.
	************************/
	afx_msg void OnEnKillfocusEditNode();

	/************************
	*	OnBnClickedButtonNdPing()
	*	OnBnClickedButtonNdPing() is called when the user selects the ping
	*	button in the node group box.  Please, investigate the implementation of each
	*	individual function for a better understanding of each command.
	************************/							
	afx_msg void OnBnClickedButtonNdPing();
	
	/************************
	*	OnBnClickedButtonNdWriteEeprom()
	*	OnBnClickedButtonNdWriteEeprom() is called when the user selects the write eeprom
	*	button in the node group box.  Please, investigate the implementation of each
	*	individual function for a better understanding of each command.
	************************/
	afx_msg void OnBnClickedButtonNdWriteEeprom();

	/************************
	*	OnEnKillfocusEditNdPageDl()
	*	OnEnKillfocusEditNdPageDl() checks the range of the page to download
	*	when focus on the page to download edit box has lost focus.
	*	PRE:	none
	*	POST:	if the value in the edit control is greater than the maximum
	*				range the value is adjusted to the maximum range value.  Likewise,
	*				if the value in the edit control is less tahn the maximum
	*				range the value is adjusted to the minimum range value.
	************************/
	afx_msg void OnEnKillfocusEditNdPageDl();
	
	/************************
	*	OnBnClickedButtonNdDlPage()
	*	OnBnClickedButtonNdDlPage() is called when the user selects the page download
	*	button in the node group box.  Please, investigate the implementation of each
	*	individual function for a better understanding of each command.
	************************/
	afx_msg void OnBnClickedButtonNdDlPage();

	/************************
	*	OnEnKillfocusEditNdTrigger()
	*	OnEnKillfocusEditNdTrigger() checks the range of the trigger id
	*	when focus on the trigger id edit box has lost focus.
	*	PRE:	none
	*	POST:	if the value in the edit control is greater than the maximum
	*				range the value is adjusted to the maximum range value.  Likewise,
	*				if the value in the edit control is less than the maximum
	*				range the value is adjusted to the minimum range value.
	************************/
	afx_msg void OnEnKillfocusEditNdTrigger();

	/************************
	*	OnBnClickedButtonNdTrigger()
	*	OnBnClickedButtonNdTrigger() is called when the user selects the trigger
	*	button in the node group box.  Please, investigate the implementation of each
	*	individual function for a better understanding of each command.
	************************/	
	afx_msg void OnBnClickedButtonNdTrigger();

	/************************
	*	OnBnClickedButtonNdSleep()
	*	OnBnClickedButtonNdSleep() is called when the user selects the sleep
	*	button in the node group box.  Please, investigate the implementation of each
	*	individual function for a better understanding of each command.
	************************/
	afx_msg void OnBnClickedButtonNdSleep();

	/************************
	*	OnBnClickedButtonNdErase()
	*	OnBnClickedButtonNdErase() is called when the user selects the erase
	*	button in the node group box.  Please, investigate the implementation of each
	*	individual function for a better understanding of each command.
	************************/	
	afx_msg void OnBnClickedButtonNdErase();

	/************************
	*	OnHelpAbout()
	*	OnHelpAbout() shows the about window
	************************/
	afx_msg void OnHelpAbout();

	/************************
	*	OnConsoleClear()
	*	OnConsoleClear() clears the console window
	************************/
	afx_msg void OnConsoleClear();

	/************************
	*	OnLbnSelchangeListConsole()
	*	OnLbnSelchangeListConsole() changes the command information box
	*	to reflect the currently selected item in the console window.
	************************/
	afx_msg void OnLbnSelchangeListConsole();

	/************************
	*	OnBnClickedButtonNdStreamSetup()
	*	OnBnClickedButtonNdStreamSetup() pops up a small streaming setup window for the
	*	user to set the channel mask and number of sweeps.  This data is then written
	*	to the corresponding node on the corresponding port.
	*	PRE:	the port number and node address are set correctly on the dialog window.
	*	POST:	if the user selected the write button on the setup window, the data has
	*			been written to the node.
	************************/
	afx_msg void OnBnClickedButtonNdStreamSetup();
	
	/************************
	*	OnBnClickedButtonNdStream()
	*	OnBnClickedButtonNdStream() initiates a stream on a node.
	************************/
	afx_msg void OnBnClickedButtonNdStream();

	/************************
	*	OnBnClickedButtonNdLongPing()
	*	OnBnClickedButtonNdLongPing() sends the long ping command to a node.
	************************/
	afx_msg void OnBnClickedButtonNdLongPing();

	/************************
	*	OnBnClickedButtonNdLowDutyCycle()
	*	OnBnClickedButtonNdLowDutyCycle() starts the low duty cycle command and reads packets from a node.
	************************/
	afx_msg void OnBnClickedButtonNdLowDutyCycle();

	/************************
	*	OnBnClickedButtonNdNodeDiscovery()
	*	OnBnClickedButtonNdNodeDiscovery() sets the base station to listen for incoming node discovery packets from nodes.
	************************/
	afx_msg void OnBnClickedButtonNdNodeDiscovery();

	/************************
	*	OnBnClickedButtonNdKill()
	*	OnBnClickedButtonNdKill() sends the Stop Node command to a node
	************************/
	afx_msg void OnBnClickedButtonNdKill();

	/************************
	*	OnBnClickedButtonNdHsStream()
	*	OnBnClickedButtonNdHsStream() sends the High Speed Stream command to a node
	************************/
	afx_msg void OnBnClickedButtonNdHsStream();

	/************************
	*	OnBnClickedBaudCheck()
	*	OnBnClickedBaudCheck() sets the baud rate of a base station
	************************/
	afx_msg void OnBnClickedBaudCheck();

	/************************
	*	OnBnClickedButtonNdSyncSampling()
	*	OnBnClickedButtonNdSyncSampling() listens for synchronized sampling packets over the air
	*
	*	Note:	This function does not send the Start Synchronized Sampling command to a node. It is
	*			assumed that a node has already been started in synchronized sampling elsewhere.
	*			Nodes need to be configured so that the appropriate TDMA slot, and other options, are
	*			written correctly to EEPROM. Failure to configure the network correctly will lead to
	*			packets colliding, resulting in a large amount of lost data.
	*
	*			Please see the Wireless Sensor Networks Data Communication Protocol for more details.
	************************/
	afx_msg void OnBnClickedButtonNdSyncSampling();

	/************************
	*	OnBnClickedBeaconEnableBtn()
	*	OnBnClickedBeaconEnableBtn() enables the beacon of a base station
	************************/
	afx_msg void OnBnClickedBeaconEnableBtn();

	/************************
	*	OnBnClickedBeaconDisableBtn()
	*	OnBnClickedBeaconDisableBtn() disabled the beacon of a base station
	************************/
	afx_msg void OnBnClickedBeaconDisableBtn();

private:
	int m_iBaudRate;	//the current baud rate of the base station
};
