// Copyright (C) 2012 MicroStrain, Inc.

/*********************
*	streamsetup.h
*	streamsetup.h holds the CStreamSetup class which allows
*	the user to setup the node for a short stream by setting
*	the number of sweeps and the channel mask.
*********************/
#include "stdafx.h"
#include "2400_sdk.h"
#include "StreamSetup.h"
#include ".\streamsetup.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// CStreamSetup dialog

IMPLEMENT_DYNAMIC(CStreamSetup, CDialog)
CStreamSetup::CStreamSetup(CWnd* pParent /*=NULL*/)
	: CDialog(CStreamSetup::IDD, pParent),
	m_usChannel_mask(0),
	m_bContinuous_streaming(FALSE),
	m_usSweeps(0)
{
}

CStreamSetup::~CStreamSetup()
{
}

void CStreamSetup::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_SPIN_SWEEPS, m_csbcSweeps);
	DDX_Control(pDX, IDC_CHECK_CONTINUOUS_STREAMING, m_checkContinuousStreaming);
}


BEGIN_MESSAGE_MAP(CStreamSetup, CDialog)
	ON_EN_KILLFOCUS(IDC_EDIT_SWEEPS, OnEnKillfocusEditSweeps)
	ON_BN_CLICKED(IDC_CHECK_CONTINUOUS_STREAMING, OnBnClickedCheckContinuousStreaming)
END_MESSAGE_MAP()


// CStreamSetup message handlers

BOOL CStreamSetup::OnInitDialog()
{
	CDialog::OnInitDialog();

	/*	Set the spin control to increment by 100 for every click	*/
	UDACCEL udAccel;
	udAccel.nInc = 100;
	m_csbcSweeps.SetAccel(1, &udAccel);
	/*	Set the range for the number of sweeps	*/
	m_csbcSweeps.SetRange32(SWEEPS_LO, SWEEPS_HI);
	/*	Set the default number of sweeps	*/
	SetDlgItemInt(IDC_EDIT_SWEEPS, SWEEPS_LO);

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

/***********************
*	OnEnKillfocusEditSweeps()
*	OnEnKillfocusEditSweeps() performs a range test on the number of
*	sweeps.  If the focus on the edit control is lost and the number of
*	sweeps is set below the low end of the range, the number of sweeps
*	is set to the low end of the range.  Likewise occurs for the high
*	end of the range.
***********************/
void CStreamSetup::OnEnKillfocusEditSweeps()
{
	int iSweeps = GetDlgItemInt(IDC_EDIT_SWEEPS);
	if (iSweeps < SWEEPS_LO)
		SetDlgItemInt(IDC_EDIT_SWEEPS, SWEEPS_LO);
	if (iSweeps > SWEEPS_HI)
		SetDlgItemInt(IDC_EDIT_SWEEPS, SWEEPS_HI);
}

/***********************
*	OnBnClickedCheckContinuousStreaming()
*	OnBnClickedCheckContinuousStreaming() evalutes the checkbox and
*   enables or disables the sweep edit control.  When continuous
*   streaming is enable the sweeps value is not used.
***********************/
void CStreamSetup::OnBnClickedCheckContinuousStreaming()
{
	/* get the state of the continuous streaming checkbox */
	m_bContinuous_streaming = m_checkContinuousStreaming.GetCheck() == BST_CHECKED;
	
	/* if continuous streaming is checked the sweep controls are disabled */
    m_csbcSweeps.EnableWindow(!m_bContinuous_streaming);
	m_csbcSweeps.GetBuddy()->EnableWindow(!m_bContinuous_streaming);
}

/***********************
*	OnOK()
*	OnOK() performs two checks on the data.  It first calculates a channel mask,
*	verifies that at least one channel is active, and saves that value.  Second,
*	the application does a quick check for the number of sweeps to be sampled
*	during streaming.  If this value is above a certain level a warning is displayed
*	to let the user know there will be a great amount of text written to the console.
***********************/
void CStreamSetup::OnOK()
{		
	/*	figure out the channel mask	*/
	m_usChannel_mask = 0;
	for (int i = 0; i < 8; i++)	//	iterate through every channel button to test if it's checked
		if (((CButton*)GetDlgItem(IDC_CHECK_CH1 + i))->GetCheck() == BST_CHECKED)
			m_usChannel_mask = (1 << i) | m_usChannel_mask;
	/*	if no channels are selected, warn the user and don't write	*/
	if (m_usChannel_mask == 0) {
		MessageBox("No channels were selected. Please, select a channel before continuing.", "Error", MB_OK | MB_APPLMODAL | MB_ICONERROR);
		return;
	}

	/*	get the number of sweeps if continuous streaming is disabled */
	if(m_bContinuous_streaming){
		m_usSweeps = 0;
	}else{
		m_usSweeps = GetDlgItemInt(IDC_EDIT_SWEEPS);
		if (m_usSweeps > SWEEP_WARN) {
			CString strMsg;
			strMsg.Format("Setting the number of sweeps beyond %i may cause a large amount of\r\ntext output.  Are you sure you want to do this?", SWEEP_WARN);
			if (MessageBox(strMsg, "Question", MB_YESNO | MB_APPLMODAL | MB_ICONQUESTION) == IDNO)
				return;
		}
	}
	CDialog::OnOK();
}
