Public Class Main

    Private Sub Main_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Sets a value indicating whether to catch calls on the wrong thread that access a control's Handle property 
        txtEvents.CheckForIllegalCrossThreadCalls = False
        txtCommandSent.CheckForIllegalCrossThreadCalls = False
        txtResponseReceived.CheckForIllegalCrossThreadCalls = False
        'populate combo
        cmbCommand.Items.Add("Ping Base Station")
        cmbCommand.Items.Add("Read Base Station EEPROM")
        cmbCommand.Items.Add("Write Base Station EEPROM")
        cmbCommand.Items.Add("Enable Beacon")
        cmbCommand.Items.Add("Disable Beacon")
        cmbCommand.SelectedItem = cmbCommand.Items.Item(0)
        'populate combo
        cmbDecHex.Items.Add("Decimal")
        cmbDecHex.Items.Add("Hex")
        cmbDecHex.SelectedItem = cmbDecHex.Items.Item(0)
        'populate event
        txtEvents.Text = "Sample code demonstrating node commands and responses..."
        'populate event
        txtEvents.Text = txtEvents.Text + vbCrLf + "We strongly recommend that you follow along in the Data Communications Protocol manual as you run each of these sample commands."
    End Sub

    Private Sub btnSendCommand_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSendCommand.Click
        'clear textboxes
        txtEvents.Text = ""
        txtCommandSent.Text = ""
        txtResponseReceived.Text = ""
        'call command function
        If cmbCommand.Text = "Ping Base Station" Then
            'call function
            Call Ping_Base_Station()
        ElseIf cmbCommand.Text = "Read Base Station EEPROM" Then
            'call function
            Call Read_Base_Station_EEPROM()
        ElseIf cmbCommand.Text = "Write Base Station EEPROM" Then
            'call function
            Call Write_Base_Station_EEPROM()
        ElseIf cmbCommand.Text = "Enable Beacon" Then
            'call function
            Call Enable_Beacon()
        ElseIf cmbCommand.Text = "Disable Beacon" Then
            'call function
            Call Disable_Beacon()
        End If
    End Sub

    Private Function Ping_Base_Station() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(NumCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 1
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 0)
        arrBytes(0) = 1
        'send command
        SerialPort1.Write(arrBytes, 0, 1)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Function EventHandler(ByVal MyEvent As String) As Boolean
        'populate event
        txtEvents.Text = txtEvents.Text + vbCrLf + MyEvent
    End Function

    Private Function ParseArray(ByVal CommandOrResponse As String, ByVal arrParse As Byte()) As Boolean
        'dim
        Dim X As Integer, strBytes As String, PadZero As String
        'seed vars
        strBytes = ""
        PadZero = ""
        'decimal or hex display
        If cmbDecHex.Text = "Decimal" Then
            'construct byte string
            For X = 0 To UBound(arrParse)
                strBytes = strBytes + Chr(32) + Str(arrParse(X))
            Next
        ElseIf cmbDecHex.Text = "Hex" Then
            'construct byte string
            For X = 0 To UBound(arrParse)
                'pad zero
                If Len(CStr(Hex(arrParse(X)))) = 1 Then
                    PadZero = "0" + CStr(Hex(arrParse(X)))
                    strBytes = strBytes + Chr(32) + PadZero
                Else
                    strBytes = strBytes + Chr(32) + CStr(Hex(arrParse(X)))
                End If
            Next
        End If
        'populate command sent or response received textbox
        If CommandOrResponse = "Command" Then
            'populate command sent
            txtCommandSent.Text = strBytes
        ElseIf CommandOrResponse = "Response" Then
            'populate response received
            txtResponseReceived.Text = strBytes
        End If
    End Function

    Private Sub SerialPort1_DataReceived(ByVal sender As System.Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles SerialPort1.DataReceived
        'set error handler
        On Error GoTo ErrorHandler
        'set variable
        UpperLimitOfArray = SerialPort1.BytesToRead - 1
        'prep array
        ReDim arrBytes(0 To UpperLimitOfArray)
        'get comm buffer
        SerialPort1.Read(arrBytes, 0, SerialPort1.BytesToRead)
        'close serial port
        SerialPort1.Close()
        'test for no byte return
        If UBound(arrBytes) = -1 Then
            'call events
            Call EventHandler("No bytes returned")
            'exit
            Exit Sub
        End If
        'populate command textbox
        Call ParseArray("Response", arrBytes)
        'test for command
        '---------------------------------------------------------
        If cmbCommand.Text = "Ping Base Station" Then
            'test for successful response
            If UBound(arrBytes) = 0 And arrBytes(0) = 1 Then
                'call events
                Call EventHandler("Response successful!")
            Else
                'call events
                Call EventHandler("Response failed!")
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Read Base Station EEPROM" Then
            'test for successful response
            If UBound(arrBytes) = 4 And arrBytes(0) = 115 And CheckSum(arrBytes) = True Then
                'calc EEPROM value
                MyEEPROM_Value = CDbl(arrBytes(1) * 256) + CDbl(arrBytes(2))
                'populate EEPROM Value numeric box
                numEEPROMValue.Value = MyEEPROM_Value
                'call events
                Call EventHandler("Response successful!")
            Else
                'call events
                Call EventHandler("Response failed!")
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Write Base Station EEPROM" Then
            'test for successful response
            If UBound(arrBytes) = 4 And arrBytes(0) = 120 And CheckSum(arrBytes) = True Then
                'calc EEPROM value
                MyEEPROM_Value = CDbl(arrBytes(1) * 256) + CDbl(arrBytes(2))
                'populate EEPROM Value numeric box
                numEEPROMValue.Value = MyEEPROM_Value
                'call events
                Call EventHandler("Response successful!")
            Else
                'call events
                Call EventHandler("Response failed!")
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Enable Beacon" Then
            'test for successful response
            If UBound(arrBytes) = 1 And arrBytes(0) = 190 And arrBytes(1) = 172 Then
                'call events
                Call EventHandler("Response successful!")
            Else
                'call events
                Call EventHandler("Response failed!")
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Disable Beacon" Then
            'test for successful response
            If UBound(arrBytes) = 1 And arrBytes(0) = 190 And arrBytes(1) = 172 Then
                'call events
                Call EventHandler("Response successful!")
            Else
                'call events
                Call EventHandler("Response failed!")
            End If
        End If

        'exit
        Exit Sub
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Sub

    Private Function CheckSum(ByVal arrParse As Byte()) As Boolean
        'dim
        Dim X As Integer, MyCheckSum As Double, TheirCheckSum As Double
        'calc MyCheckSum
        For X = 1 To UBound(arrParse) - 2
            MyCheckSum = MyCheckSum + CDbl(arrParse(X))
        Next
        'calc TheirCheckSum
        TheirCheckSum = CDbl(arrParse(UBound(arrParse) - 1) * 256) + CDbl(arrParse(UBound(arrParse)))
        'test checksums
        If MyCheckSum Mod 65536 = TheirCheckSum Then
            CheckSum = True
        Else
            CheckSum = False
        End If
    End Function

    Private Function Read_Base_Station_EEPROM() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(NumCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 5
        'open serial port
        SerialPort1.Open()
        'dim
        Dim MyCS As Double
        'construct command using decimals
        ReDim arrBytes(0 To 4)
        arrBytes(0) = 115
        arrBytes(1) = numEEPROMAddress.Value \ 256
        arrBytes(2) = numEEPROMAddress.Value Mod 256
        MyCS = arrBytes(1) + arrBytes(2)
        arrBytes(3) = MyCS \ 256
        arrBytes(4) = MyCS Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 5)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Function Write_Base_Station_EEPROM() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(NumCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 5
        'open serial port
        SerialPort1.Open()
        'dim
        Dim MyCS As Double
        'construct command using decimals
        ReDim arrBytes(0 To 6)
        arrBytes(0) = 120
        arrBytes(1) = numEEPROMAddress.Value \ 256
        arrBytes(2) = numEEPROMAddress.Value Mod 256
        arrBytes(3) = numEEPROMValue.Value \ 256
        arrBytes(4) = numEEPROMValue.Value Mod 256
        MyCS = CDbl(arrBytes(1)) + CDbl(arrBytes(2)) + CDbl(arrBytes(3)) + CDbl(arrBytes(4))
        arrBytes(5) = MyCS \ 256
        arrBytes(6) = MyCS Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 7)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function
    Private Function Enable_Beacon() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(NumCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 2
        'open serial port
        SerialPort1.Open()
        'dim
        Dim MyCS As Double
        'construct command using decimals
        ReDim arrBytes(0 To 5)
        arrBytes(0) = 190
        arrBytes(1) = 172
        arrBytes(2) = 0
        arrBytes(3) = 0
        arrBytes(4) = 0
        arrBytes(5) = 0
        'send command
        SerialPort1.Write(arrBytes, 0, 6)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Function Disable_Beacon() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(NumCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 2
        'open serial port
        SerialPort1.Open()
        'dim
        Dim MyCS As Double
        'construct command using decimals
        ReDim arrBytes(0 To 5)
        arrBytes(0) = 190
        arrBytes(1) = 172
        arrBytes(2) = 255
        arrBytes(3) = 255
        arrBytes(4) = 255
        arrBytes(5) = 255
        'send command
        SerialPort1.Write(arrBytes, 0, 6)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Sub numEEPROMAddress_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numEEPROMAddress.ValueChanged
        'increment EEPROM address so that it is always an even number
        If (numEEPROMAddress.Value And 1) <> 0 Then
            numEEPROMAddress.Value = 0
        End If
    End Sub
End Class
